package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.PmReceive;
import com.usky.iot.domain.PmWorkContent;
import com.usky.iot.domain.PmWorkReport;
import com.usky.iot.mapper.*;
import com.usky.iot.service.PmProjectService;
import com.usky.iot.service.PmWorkContentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.service.vo.ProjectWorkTimeVO;
import com.usky.iot.service.vo.UserWorkTimeVO;
import com.usky.iot.service.vo.WorkHoursStatisticsVO;
import com.usky.system.domain.SysUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 工作内容表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Service
public class PmWorkContentServiceImpl extends AbstractCrudService<PmWorkContentMapper, PmWorkContent> implements PmWorkContentService {

    @Autowired
    private PmWorkReportMapper pmWorkReportMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private PmProjectService pmProjectService;

    @Autowired
    private PmReceiveMapper pmReceiveMapper;

    @Override
    public void deleteContent(Integer reportId) {
        LambdaQueryWrapper<PmWorkContent> deleteWrapper = Wrappers.lambdaQuery();
        deleteWrapper.eq(PmWorkContent::getReportId, reportId);
        baseMapper.delete(deleteWrapper);
    }

    @Override
    public List<PmWorkContent> projectQuery(String startDate, String endDate, String projectName, Integer projectAscription, Integer reportId) {
        if (reportId != null && reportId != 0) {
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getProjectId, PmWorkContent::getProjectName)
                    .eq(PmWorkContent::getReportId, reportId)
                    .groupBy(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
            List<PmWorkContent> pmWorkContentList = this.list(contentLambdaQuery);
            return pmWorkContentList;
        }
        LocalDate start = LocalDate.parse(startDate);
        LocalDate end = LocalDate.parse(endDate);
        String subQueryReports = "SELECT id FROM pm_work_report WHERE report_date BETWEEN '" + start + "' AND '" + end + "' AND tenant_id = " + SecurityUtils.getTenantId();
        String subQueryReport = "SELECT id FROM pm_work_report WHERE FIND_IN_SET('" + SecurityUtils.getUserId() + "', cc_to) > 0 AND tenant_id = " + SecurityUtils.getTenantId();
        String sentOut = "SELECT id FROM pm_work_report WHERE create_by = '" + SecurityUtils.getUsername() + "' AND tenant_id = " + SecurityUtils.getTenantId();
        String subQueryProjects = "SELECT id FROM pm_project WHERE project_head = '" + SecurityUtils.getUserId() + "' AND tenant_id = " + SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getProjectId, PmWorkContent::getProjectName)
                .inSql(PmWorkContent::getReportId, subQueryReports);
        if (StringUtils.isNotBlank(projectName)) {
            contentLambdaQuery.like(PmWorkContent::getProjectName, projectName);
        }
        if (projectAscription == 1) {
            contentLambdaQuery.inSql(PmWorkContent::getProjectId, subQueryProjects);
        } else if (projectAscription == 2) {
            contentLambdaQuery.inSql(PmWorkContent::getReportId, subQueryReport);
        } else if (projectAscription == 3) {
            contentLambdaQuery.inSql(PmWorkContent::getReportId, sentOut);
        }
        contentLambdaQuery.groupBy(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
        List<PmWorkContent> pmWorkContentList = this.list(contentLambdaQuery);
        return pmWorkContentList;
    }

    @Override
    public List<PmWorkReport> workReportQuery(String startDate, String endDate, String projectName, Integer projectAscription, Integer reportId) {
        List<PmWorkReport> reportList = new ArrayList<>();
        List<PmWorkContent> pmWorkContentList = new ArrayList<>();
        List<Long> userIds = new ArrayList<>();
        if (reportId != null && reportId != 0) {
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                    PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .eq(PmWorkReport::getId, reportId);
            PmWorkReport report = pmWorkReportMapper.selectOne(reportQuery);
            LambdaQueryWrapper<PmWorkContent> contentQuery = Wrappers.lambdaQuery();
            contentQuery.select(PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName)
                    .eq(PmWorkContent::getReportId, reportId);
            pmWorkContentList = this.list(contentQuery);
            report.setWorkContents(pmWorkContentList);
            reportList.add(report);
            for (PmWorkReport c : reportList) {
                userIds.add(c.getSubmitterId());
            }
        } else {
            LocalDate start = LocalDate.parse(startDate);
            LocalDate end = LocalDate.parse(endDate);
            String subQueryReports = "SELECT id FROM pm_work_report WHERE report_date BETWEEN '" + start + "' AND '" + end + "' AND tenant_id = " + SecurityUtils.getTenantId();
            String subQueryReport = "SELECT id FROM pm_work_report WHERE FIND_IN_SET('" + SecurityUtils.getUserId() + "', cc_to) > 0 AND tenant_id = " + SecurityUtils.getTenantId();
            String subQueryProjects = "SELECT id FROM pm_project WHERE project_head = '" + SecurityUtils.getUserId() + "' AND tenant_id = " + SecurityUtils.getTenantId();
            String sentOut = "SELECT id FROM pm_work_report WHERE create_by = '" + SecurityUtils.getUsername() + "' AND tenant_id = " + SecurityUtils.getTenantId();
            String reportIdList = "SELECT report_id FROM pm_work_content WHERE project_name = '" + projectName + "' AND tenant_id = " + SecurityUtils.getTenantId();
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName)
                    .inSql(PmWorkContent::getReportId, subQueryReports);
            if (projectAscription == 1) {
                contentLambdaQuery.inSql(PmWorkContent::getProjectId, subQueryProjects);
            } else if (projectAscription == 2) {
                contentLambdaQuery.inSql(PmWorkContent::getReportId, subQueryReport);
            } else if (projectAscription == 3) {
                contentLambdaQuery.inSql(PmWorkContent::getReportId, sentOut);
            }
            if (StringUtils.isNotBlank(projectName)) {
                contentLambdaQuery.inSql(PmWorkContent::getReportId, reportIdList);
            }
            pmWorkContentList = this.list(contentLambdaQuery);
            if (pmWorkContentList.isEmpty()) {
                return reportList;
            }
            List<Integer> reportIds = new ArrayList<>();
            for (PmWorkContent a : pmWorkContentList) {
                reportIds.add(a.getReportId());
            }
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                    PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .in(PmWorkReport::getId, reportIds)
                    .orderByDesc(PmWorkReport::getCreateTime);
            reportList = pmWorkReportMapper.selectList(reportQuery);
            LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
            statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                    .eq(PmReceive::getReceiverId, SecurityUtils.getUserId())
                    .eq(PmReceive::getTenantId, SecurityUtils.getTenantId())
                    .in(PmReceive::getReportId, reportIds);
            List<PmReceive> receiveList = pmReceiveMapper.selectList(statusQuery);
            for (PmWorkReport c : reportList) {
                List<PmWorkContent> contents = new ArrayList<>();
                for (PmWorkContent b : pmWorkContentList) {
                    if (b.getReportId().equals(c.getId())) {
                        contents.add(b);
                    }
                }
                c.setWorkContents(contents);
                if (!receiveList.isEmpty()) {
                    for (PmReceive receive : receiveList) {
                        if (receive.getReportId().equals(c.getId())) {
                            c.setReadFlag(receive.getReadFlag());
                        }
                    }
                }
                userIds.add(c.getSubmitterId());
            }
        }
        List<SysUser> nickNames = nickNames(userIds);
        for (PmWorkReport d : reportList) {
            for (SysUser e : nickNames) {
                if (e.getUserId().equals(d.getSubmitterId())) {
                    d.setCreateBy(e.getNickName());
                }
            }
            List<Long> ids = Optional.ofNullable(d.getCcTo())
                    .map(ccTo -> Arrays.stream(ccTo.split(","))
                            .map(Long::parseLong)
                            .collect(Collectors.toList()))
                    .orElse(Collections.emptyList());
            List<SysUser> nickNameList = nickNames(ids);
            StringBuilder ccToBuilder = new StringBuilder();
            Set<Long> addedUserIds = new HashSet<>();
            for (Long id : ids) {
                for (SysUser user : nickNameList) {
                    if (user.getUserId().equals(id) && !addedUserIds.contains(id)) {
                        if (ccToBuilder.length() > 0) {
                            ccToBuilder.append(", ");
                        }
                        ccToBuilder.append(user.getNickName());
                        addedUserIds.add(id);
                        break;
                    }
                }
            }
            String ccTo = ccToBuilder.toString();
            d.setCcTo(ccTo);
        }
        return reportList;
    }

    @Override
    public List<SysUser> nickNames(List<Long> userIds) {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0)
                .in(SysUser::getUserId, userIds);
        return sysUserMapper.selectList(usersQuery);
    }

    @Override
    public List<SysUser> allNickName() {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0);
        return sysUserMapper.selectList(usersQuery);
    }

    @Override
    public CommonPage<PmWorkReport> reportPage(Integer projectAscription, Integer pageNum, Integer pageSize) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long userId = SecurityUtils.getUserId();
        List<Long> userIds = new ArrayList<>();
        Page<PmWorkReport> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName)
                .apply("report_id IN (SELECT id FROM pm_work_report WHERE tenant_id = '" + tenantId + "')")
                .apply("tenant_id = '" + tenantId + "'");

        switch (projectAscription) {
            case 1:
                contentLambdaQuery.apply("project_id IN (SELECT id FROM pm_project WHERE project_head = '" + userId + "' AND tenant_id = '" + tenantId + "')");
                break;
            case 2:
                contentLambdaQuery.apply("report_id IN (SELECT id FROM pm_work_report WHERE FIND_IN_SET('" + userId + "', cc_to) > 0 AND tenant_id = '" + tenantId + "')");
                break;
            case 3:
                contentLambdaQuery.apply("create_by ='" + userId + "' AND tenant_id = '" + tenantId + "'");
                break;
        }

        List<PmWorkContent> pmWorkContentList = this.list(contentLambdaQuery);
        Set<Integer> reportIds = pmWorkContentList.stream().map(PmWorkContent::getReportId).collect(Collectors.toSet());

        if (reportIds.isEmpty()) {
            return new CommonPage<>(new ArrayList<>(), 0, pageSize, pageNum);
        }

        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                .in(PmWorkReport::getId, reportIds)
                .orderByDesc(PmWorkReport::getCreateTime);

        IPage<PmWorkReport> reportPage = pmWorkReportMapper.selectPage(new Page<>(pageNum, pageSize), reportQuery);
        List<PmWorkReport> reportList = reportPage.getRecords();

        LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
        statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                .eq(PmReceive::getReceiverId, userId)
                .eq(PmReceive::getTenantId, tenantId)
                .in(PmReceive::getReportId, reportIds);

        List<PmReceive> receiveList = pmReceiveMapper.selectList(statusQuery);

        Map<Integer, Integer> reportReadFlags = receiveList.stream()
                .collect(Collectors.toMap(PmReceive::getReportId, PmReceive::getReadFlag));

        reportList.forEach(report -> {
            List<PmWorkContent> contents = pmWorkContentList.stream()
                    .filter(content -> content.getReportId().equals(report.getId()))
                    .collect(Collectors.toList());
            report.setWorkContents(contents);
            userIds.add(report.getSubmitterId());
            if (reportReadFlags.containsKey(report.getId())) {
                report.setReadFlag(reportReadFlags.get(report.getId()));
            }
        });

        List<SysUser> nickNames = nickNames(userIds);
        Map<Long, String> userNicknameMap = nickNames.stream()
                .collect(Collectors.toMap(SysUser::getUserId, SysUser::getNickName));

        reportList.forEach(report -> {
            report.setCreateBy(userNicknameMap.get(report.getSubmitterId()));
        });

        return new CommonPage<>(reportList, reportPage.getTotal(), pageSize, pageNum);
    }


    @Override
    public List<Object> workHourStatistic(Long userId, Integer projectId, Integer filter, String startDate, String endDate, Integer workerOrProject) {
        List<ProjectWorkTimeVO> projectWorkTime = new ArrayList<>();
        List<UserWorkTimeVO> userWorkTimeVO = new ArrayList<>();
        Integer tenantId = SecurityUtils.getTenantId();
        List<Object> returnList = new ArrayList<>();
        LocalDate start = LocalDate.parse(startDate);
        LocalDate end = LocalDate.parse(endDate);
        //找出租户下所有项目
        List<Map<String, Object>> projectIdName = pmProjectService.projectIdName(projectId);
        List<String> pmProjectName = projectIdName.stream()
                .map(map -> (String) map.get("name"))
                .collect(Collectors.toList());
        //所有人员id和nickName
        List<Map<String, Object>> users = pmProjectService.projectUsers(userId);
        List<String> nickName = new ArrayList<>();
        for (Map<String, Object> map : users) {
            Object name = map.get("name");
            nickName.add((String) name);
        }
        //统计userId每个项目所需工时
        List<WorkHoursStatisticsVO> userAndProject = baseMapper.workHoursStatistics(start, end, userId, tenantId);
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        List<WorkHoursStatisticsVO> userAndProjectD = baseMapper.statisticsDept(start, end, userId, tenantId, deptId);
        if (workerOrProject != 1) {
            returnList.add(nickName);
            //重组返回数据结构
            for (String name : pmProjectName) {
                ProjectWorkTimeVO projectWorkTimeVO = new ProjectWorkTimeVO();
                projectWorkTimeVO.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : nickName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProject) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (projectName.equals(name) && userNickName.equals(currentNickName)) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    projectWorkTimeVO.setWorkTime(workTime);
                    projectWorkTime.add(projectWorkTimeVO);
                }
            }
            returnList.add(projectWorkTime);
        } else {
            returnList.add(pmProjectName);
            //重组返回数据结构
            for (String uName : nickName) {
                String name = uName;
                UserWorkTimeVO userWorkTimeVO2 = new UserWorkTimeVO();
                userWorkTimeVO2.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : pmProjectName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProjectD) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (userNickName.equals(name) && projectName.equals(currentNickName)) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    userWorkTimeVO2.setWorkTime(workTime);
                    userWorkTimeVO.add(userWorkTimeVO2);
                }
            }
            returnList.add(userWorkTimeVO);
        }
        return returnList;
    }

}
